/*! Backbone.Marionette.Handlebars - v0.1.0
------------------------------
Build @ 2012-05-24
Documentation and Full License Available at:
http://asciidisco.github.com/Backbone.Marionette.Handlebars/index.html
git://github.com/asciidisco/Backbone.Marionette.Handlebars.git
Copyright (c) 2012 Sebastian Golasch <public@asciidisco.com>

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the

Software is furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.*/

(function (root, define, require, exports, module, factory, undef) {
    'use strict';
    if (typeof exports === 'object') {
        // Node. Does not work with strict CommonJS, but
        // only CommonJS-like enviroments that support module.exports,
        // like Node.
        module.exports = factory(require('underscore'), require('backbone'), require('backbone.marionette'));
    } else if (typeof define === 'function' && define.amd) {
        // AMD. Register as an anonymous module.
        define(['underscore', 'backbone', 'backbone.marionette'], function (_, Backbone) {
            // Check if we use the AMD branch of Backbone
            _ = _ === undef ? root._ : _;
            Backbone = Backbone === undef ? root.Backbone : Backbone;
            return (root.returnExportsGlobal = factory(_, Backbone, root));
        });
    } else {
        // Browser globals
        root.returnExportsGlobal = factory(root._, root.Backbone);
    }
}(this, this.define, this.require, this.exports, this.module, function (_, Backbone, root, undef) {
    'use strict';     
    var oldGetTemplateSelector, oldTemplateCacheGet, oldRenderTemplate;

    oldGetTemplateSelector = Backbone.Marionette.View.prototype.getTemplateSelector;
    Backbone.Marionette.View.prototype.getTemplateSelector = function () {
        var template;

        // Get the template from `this.options.template` or
        // `this.template`. The `options` takes precedence.
        if (this.options && this.options.template) {
            template = this.options.template;
        } else {
            template = this.template;
        }

        // check if it's a handlebars template
        if (_.isObject(template) && template.type === 'handlebars') {
            return template;
        }

        return _.bind(oldGetTemplateSelector, this)();
    };

    oldTemplateCacheGet = Backbone.Marionette.TemplateCache.get;
    Backbone.Marionette.TemplateCache.get = function (template) {
        // check if it's a handlebars template
        if (_.isObject(template) && template.type === 'handlebars') {
            return template;
        }

        return _.bind(oldTemplateCacheGet, this)(template);
    };

    oldRenderTemplate = Backbone.Marionette.Renderer.renderTemplate;
    Backbone.Marionette.Renderer.renderTemplate = function (template, data) {
        if (template !== null) {
            return template.template(data, template.options);
        }
    };

    return Backbone.Marionette;
}));    